----------------------------------------------------------------------------------
-- POLYTECHNIQUE MONTREAL
-- ELE3311 - Systemes logiques programmables 
-- 
-- Module Name:    decod_1p4_tb
-- Description:    Banc d'essai du decodeur 1 parmi 4 avec enable
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity decod_1p4_tb is
end decod_1p4_tb;


architecture testbench of decod_1p4_tb is

  component decod_1p4
    port (
      clk_i                 : in    std_logic;
      rst_i                 : in    std_logic;  -- synchrone
      en_i                  : in    std_logic;  -- enable
      addr_i                : in    std_logic_vector(1 downto 0);
      decod_o               : out   std_logic_vector(3 downto 0)
    );
  end component;

  signal clk                      : std_logic := '0';
  signal rst                      : std_logic := '1';

  signal en                       : std_logic := '0';
  signal addr                     : std_logic_vector(1 downto 0) := (others => '0');
  signal decod                    : std_logic_vector(3 downto 0);

  -- Periode d'horloge
  constant clk_period             : time := 10 ns; -- 100 MHz

  -- Verification
  signal clk_en                   : boolean := true;

  begin

    -- Instantiation du composant a verifier - Unit Under Test (UUT)
    UUT: decod_1p4
    port map (
      rst_i                 => rst,
      clk_i                 => clk,
      en_i                  => en,
      addr_i                => addr,
      decod_o               => decod
    );
  
    -- Generation de l'horloge et de la remise a zero
    clk    <= not clk after clk_period/2 when clk_en else
              '0';
  
    rst    <= '1', '0' after clk_period*10;
  
  
    -- Stimulus process
    driver: process
    begin
      -- On synchronise les signaux avec le front descendant de l'horloge
      -- afin de respecter les temps de preparation et de maintien (setup/hold times) 
      for i in 0 to 20 loop
        wait until falling_edge(clk);
      end loop;

      -- *** A completer DEBUT ***
      addr <= "00";
      en <= '0';
      wait for clk_period*10;

      en <= '1';

      addr <= "00";
      wait for clk_period*10;

      addr <= "01";
      wait for clk_period*10;

      addr <= "10";
      wait for clk_period*10;

      addr <= "11";
      wait for clk_period*10;

      -- *** A completer FIN ***
  
      for i in 0 to 100 loop
        wait until falling_edge(clk);
      end loop;
  
      -- Fin de la sequence de test
      -- Desactivation de l'horloge
      clk_en    <= false;
      wait;
     end process;
  
  end testbench;